import { EntryData, NamespaceData, NAMESPACES_PROGRAM } from "@cardinal/namespaces";
import { AnchorProvider, IdlTypes, Program } from "@project-serum/anchor";
import { Commitment, ConfirmedTransactionMeta, Finality, Keypair, Message, PublicKey } from "@solana/web3.js";
import { AnchorSdk, BigInstructionResult, InstructionResult, TypedAccountParser } from "@strata-foundation/spl-utils";
import BN from "bn.js";
import LitJsSdk from "lit-js-sdk";
import { CaseInsensitiveMarkerV0, ChatIDL, ChatV0, DelegateWalletV0, NamespacesV0, PermissionType, PostAction, ProfileV0, SettingsV0 } from "./generated/chat";
export * from "./generated/chat";
export * from "./shdw";
export * from "./lit";
interface SymKeyInfo {
    encryptedSymKey: string;
    symKey: string;
    timeMillis: number;
}
interface ISymKeyStorage {
    setSymKeyToUse(mintOrCollection: PublicKey, amount: number, key: SymKeyInfo): void;
    getSymKeyToUse(mintOrCollection: PublicKey, amount: number): SymKeyInfo | null;
    setSymKey(encrypted: string, unencrypted: string): void;
    getSymKey(encrypted: string): string | null;
    getTimeSinceLastSet(mintOrCollection: PublicKey, amount: number): number | null;
}
export declare class LocalSymKeyStorage implements ISymKeyStorage {
    readonly url: string;
    constructor(url: string);
    setSymKey(encrypted: string, unencrypted: string): void;
    getSymKey(encrypted: string): string | null;
    private getKey;
    setSymKeyToUse(mintOrCollection: PublicKey, amount: number, symKey: SymKeyInfo): void;
    getTimeSinceLastSet(mintOrCollection: PublicKey, amount: number): number | null;
    getSymKeyToUse(mintOrCollection: PublicKey, amount: number): SymKeyInfo | null;
}
export declare function importSymmetricKey(symmKey: BufferSource): Promise<CryptoKey>;
export declare function exportSymmetricKey(symmKey: CryptoKey): Promise<ArrayBuffer>;
export declare enum MessageType {
    Text = "text",
    Html = "html",
    Gify = "gify",
    Image = "image",
    React = "react"
}
export interface INamespaces extends NamespacesV0 {
    publicKey: PublicKey;
    chat: NamespaceData;
    user: NamespaceData;
}
export interface ReactMessage {
    referenceMessageId: string;
    emoji: string;
}
export interface TextMessage {
    text: string;
}
export interface HtmlMessage {
    html: string;
    encryptedAttachments: {
        name: string;
        file: string;
    }[];
}
export interface ImageMessage {
    attachments: {
        name: string;
        file: string;
    }[];
    encryptedAttachments: {
        name: string;
        file: string;
    }[];
}
export interface GifyMessage {
    gifyId: string;
}
export interface IMessageContent extends Partial<ReactMessage>, Partial<TextMessage>, Partial<HtmlMessage>, Partial<ImageMessage>, Partial<GifyMessage> {
    type: MessageType;
    referenceMessageId?: string;
}
export interface IDecryptedMessageContent extends IMessageContent {
    decryptedAttachments?: {
        name: string;
        file: Blob;
    }[];
}
export interface ISendMessageContent extends IMessageContent {
    fileAttachments?: {
        name: string;
        file: File;
    }[];
}
export interface IDelegateWallet extends DelegateWalletV0 {
    publicKey: PublicKey;
}
export interface IChat extends ChatV0 {
    publicKey: PublicKey;
}
export interface IEntry extends EntryData {
    publicKey: PublicKey;
    mint: PublicKey;
}
export interface IProfile extends ProfileV0 {
    publicKey: PublicKey;
}
export interface ISettings extends SettingsV0 {
    getDelegateWalletSeed(): Promise<string>;
}
export interface ICaseInsensitiveMarker extends CaseInsensitiveMarkerV0 {
    publicKey: PublicKey;
}
declare type MessagePartV0 = IdlTypes<ChatIDL>["MessagePartV0"];
export interface IMessagePart extends MessagePartV0 {
    txid: string;
    blockTime: number;
    sender: PublicKey;
    chatKey: PublicKey;
}
export interface IMessage {
    type: MessageType;
    id: string;
    txids: string[];
    startBlockTime: number;
    endBlockTime: number;
    readPermissionAmount: BN;
    referenceMessageId: string | null;
    encryptedSymmetricKey: string;
    content: string;
    getDecodedMessage(): Promise<IDecryptedMessageContent | undefined>;
    sender: PublicKey;
    chatKey: PublicKey;
    parts: IMessagePart[];
}
export interface InitializeChatArgs {
    payer?: PublicKey;
    /** The admin of this chat instance. **Default:** This wallet */
    admin?: PublicKey;
    /** The unique shortname of the chat. This is a cardinal certificate NFT */
    identifierCertificateMint: PublicKey;
    /** Human readable name for the chat */
    name: string;
    /** The mint you need to read this chat */
    readPermissionKey: PublicKey;
    /** The mint you need to post to this chat */
    postPermissionKey: PublicKey;
    /** The gating mechanism, part of an NFT collection or just holds the token. **Default:** Token */
    readPermissionType?: PermissionType;
    /** The gating mechanism, part of an NFT collection or just holds the token **Default:** Token */
    postPermissionType?: PermissionType;
    /** The number of tokens needed to post to this chat. **Default:** 1 */
    postPermissionAmount?: number | BN;
    /** The action to take when posting. **Default:** hold */
    postPermissionAction?: PostAction;
    /** Amount of read permission mint required to read this chat by default. **Default:** 1 */
    defaultReadPermissionAmount?: any;
    /** The destination to pay to on post */
    postPayDestination?: PublicKey;
    imageUrl?: string;
    metadataUrl?: string;
}
export interface InitializeProfileArgs {
    payer?: PublicKey;
    /** The owner of this profile. **Default:** the current wallet */
    ownerWallet?: PublicKey;
    /** The unique shortname of the user. This is a cardinal certificate NFT */
    identifierCertificateMint: PublicKey;
    /** Useful when metadata is being created in the same instruction set, short circuit call to get metadata for the identifier. **Default:** metadata of certificate mint name */
    identifier?: string;
    imageUrl?: string;
    metadataUrl?: string;
}
export interface InitializeSettingsArgs {
    payer?: PublicKey;
    /** The owner of this settings. **Default:** the current wallet */
    ownerWallet?: PublicKey;
    settings: {
        delegateWalletSeed: string;
    };
}
export interface InitializeDelegateWalletArgs {
    payer?: PublicKey;
    /** The owning wallet of the delegate. **Default:** the current wallet */
    ownerWallet?: PublicKey;
    /** The delegate wallet to use. **Default:** from keypair */
    delegateWallet?: PublicKey;
    /** The delegate keypair. **Default:** Generate one */
    delegateWalletKeypair?: Keypair;
}
export interface SendMessageArgs {
    payer?: PublicKey;
    /** The chat to send to */
    chat: PublicKey;
    /** The message to send */
    message: ISendMessageContent;
    /** The amount of tokens needed to read. **Default:** from chat */
    readPermissionAmount?: number;
    /** Lit protocol conditions, **Default:** The chatroom default */
    accessControlConditions?: any;
    /** If using a delegate wallet, will send and sign. **Defualt:** delegateWalletKeypair.publicKey */
    delegateWallet?: PublicKey;
    /** If using a delegate wallet, will send and sign */
    delegateWalletKeypair?: Keypair;
    /** The ownerWallet of the sender. **Default:** this wallet*/
    sender?: PublicKey;
    /** Should we encrypt this message using lit protocol? */
    encrypted?: boolean;
    /** If you need an nft to post the message, this should be the mint of the qualifying nft held by the sender */
    nftMint?: PublicKey;
}
export declare enum IdentifierType {
    Chat = "chat",
    User = "me"
}
export interface ClaimIdentifierArgs {
    payer?: PublicKey;
    /** The wallet to own this. **Default:** this wallet */
    owner?: PublicKey;
    type: IdentifierType;
    identifier: string;
}
export declare class ChatSdk extends AnchorSdk<ChatIDL> {
    litClient: LitJsSdk;
    litAuthSig: any | undefined;
    chain: string;
    authingLit: Promise<void> | null;
    symKeyStorage: ISymKeyStorage;
    symKeyFetchCache: Record<string, Promise<Uint8Array | undefined>>;
    litJsSdk: LitJsSdk;
    namespacesProgram: Program<NAMESPACES_PROGRAM>;
    conditionVersion: number;
    _namespaces: INamespaces | null;
    _namespacesPromise: Promise<INamespaces> | null;
    static ID: PublicKey;
    get isLitAuthed(): boolean;
    _litAuth(): Promise<void>;
    litAuth(): Promise<void | null>;
    getNamespace(namespace: PublicKey): Promise<NamespaceData>;
    static init(provider: AnchorProvider, chatProgramId?: PublicKey): Promise<ChatSdk>;
    constructor({ provider, program, litClient, namespacesProgram, symKeyStorage, }: {
        provider: AnchorProvider;
        program: Program<ChatIDL>;
        litClient: typeof LitJsSdk;
        namespacesProgram: Program<NAMESPACES_PROGRAM>;
        symKeyStorage?: ISymKeyStorage;
    });
    entryDecoder: TypedAccountParser<IEntry>;
    chatDecoder: TypedAccountParser<IChat>;
    delegateWalletDecoder: TypedAccountParser<IDelegateWallet>;
    profileDecoder: TypedAccountParser<IProfile>;
    settingsDecoder: TypedAccountParser<ISettings>;
    caseInsensitiveMarkerDecoder: TypedAccountParser<ICaseInsensitiveMarker>;
    getChat(chatKey: PublicKey): Promise<IChat | null>;
    getProfile(profileKey: PublicKey): Promise<IProfile | null>;
    getSettings(settingsKey: PublicKey): Promise<ISettings | null>;
    getCaseInsensitiveMarker(caseInsensitiveMarkerKey: PublicKey): Promise<ICaseInsensitiveMarker | null>;
    getMessagesFromParts(parts: IMessagePart[], ignorePartial?: boolean): Promise<IMessage[]>;
    _getSymmetricKey(encryptedSymmetricKey: string, accessControlConditions: any): Promise<Uint8Array | undefined>;
    getSymmetricKey(encryptedSymmetricKey: string, accessControlConditions: any): Promise<Uint8Array | undefined>;
    getMessageFromParts(parts: IMessagePart[], ignorePartial?: boolean): Promise<IMessage | undefined>;
    getMessagePartsFromInflatedTx({ transaction, txid, meta, blockTime, }: {
        transaction: {
            message: Message;
            signatures: string[];
        };
        txid: string;
        meta?: ConfirmedTransactionMeta | null;
        blockTime?: number | null;
    }): Promise<IMessagePart[]>;
    getMessagePartsFromTx(txid: string): Promise<IMessagePart[]>;
    static chatKey(identifierCertificateMint: PublicKey, programId?: PublicKey): Promise<[PublicKey, number]>;
    static caseInsensitiveMarkerKey(namespace: PublicKey, identifier: string, programId?: PublicKey): Promise<[PublicKey, number]>;
    static entryKey(namespaceId: PublicKey, identifier: string): Promise<[PublicKey, number]>;
    static delegateWalletKey(delegateWallet: PublicKey, programId?: PublicKey): Promise<[PublicKey, number]>;
    static profileKey(wallet: PublicKey, programId?: PublicKey): Promise<[PublicKey, number]>;
    static settingsKey(wallet: PublicKey, programId?: PublicKey): Promise<[PublicKey, number]>;
    static namespacesKey(programId?: PublicKey): Promise<[PublicKey, number]>;
    initializeNamespacesInstructions(): Promise<InstructionResult<null>>;
    initializeNamespaces(): Promise<null>;
    _getNamespaces(): Promise<INamespaces>;
    getNamespaces(): Promise<INamespaces>;
    /**
     * Attempt to claim the identifier. If the identifier entry already exists, attempt to approve/claim.
     * @param param0
     * @returns
     */
    claimIdentifierInstructions({ payer, owner, identifier, type, }: ClaimIdentifierArgs): Promise<BigInstructionResult<{
        certificateMint: PublicKey;
    }>>;
    claimIdentifier(args: ClaimIdentifierArgs, commitment?: Finality): Promise<{
        certificateMint: PublicKey;
    }>;
    initializeChatInstructions({ payer, identifierCertificateMint, name, readPermissionKey, postPermissionKey, postPermissionAction, postPayDestination, postPermissionAmount, defaultReadPermissionAmount, imageUrl, metadataUrl, readPermissionType, postPermissionType, }: InitializeChatArgs): Promise<InstructionResult<{
        chat: PublicKey;
    }>>;
    initializeChat(args: InitializeChatArgs, commitment?: Commitment): Promise<{
        chat: PublicKey;
    }>;
    initializeProfileInstructions({ payer, ownerWallet, identifierCertificateMint, identifier, imageUrl, metadataUrl, }: InitializeProfileArgs): Promise<InstructionResult<{
        walletProfile: PublicKey;
    }>>;
    initializeProfile(args: InitializeProfileArgs, commitment?: Commitment): Promise<{
        walletProfile: PublicKey;
    }>;
    initializeSettingsInstructions({ payer, ownerWallet, settings, }: InitializeSettingsArgs): Promise<InstructionResult<{
        settings: PublicKey;
    }>>;
    initializeSettings(args: InitializeSettingsArgs, commitment?: Commitment): Promise<{
        settings: PublicKey;
    }>;
    initializeDelegateWalletInstructions({ payer, ownerWallet, delegateWalletKeypair, delegateWallet, }: InitializeDelegateWalletArgs): Promise<InstructionResult<{
        delegateWallet: PublicKey;
        delegateWalletKeypair?: Keypair;
    }>>;
    initializeDelegateWallet(args: InitializeDelegateWalletArgs, commitment?: Commitment): Promise<{
        delegateWallet: PublicKey;
        delegateWalletKeypair?: Keypair;
    }>;
    sendMessageInstructions({ payer, sender, chat, message: rawMessage, readPermissionAmount, delegateWallet, delegateWalletKeypair, encrypted, nftMint, }: SendMessageArgs): Promise<BigInstructionResult<{
        messageId: string;
    }>>;
    sendMessage(args: SendMessageArgs, commitment?: Finality): Promise<{
        txids?: string[];
        messageId: string;
    }>;
}
//# sourceMappingURL=index.d.ts.map